﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MotorBoat
{
    /// <summary>.
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    public class DrawningMotorboat
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityMotorBoat? EntityMotorBoat { get; private set; }

        /// <summary>
        /// Ширина окна
        /// </summary>
        private int _pictureWight = 800;

        /// <summary>
        /// Высота окна
        /// </summary>
        private int _pictureHeight = 450;

        /// <summary>
        /// Левая координата прорисовки катера
        /// </summary>
        private int _startPosX = 0;

        /// <summary>
        /// Правая координата прорисовки катера
        /// </summary>
        private int _startPosY = 0;

        /// <summary>
        /// Ширина прорисовки автомобиля
        /// </summary>
        private readonly int _boatWight = 120;

        /// <summary>
        /// Высота прорисовки автомобиля
        /// </summary>
        private readonly int _boatHeight = 80;

        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Цвет корпуса</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="wight">Ширина картинки</param>
        /// <param name="height">Выслота картинки</param>
        /// <returns>true - объект создан, false - проверка не пройдена, нельзя создать объект в этих размерах</returns>
        public bool Init(int speed, double weight, Color bodyColor, Color AdditionalColor, int width, int height)
        {
            if (width <= _boatWight || height <= _boatHeight) 
                return false; 
            _pictureWight = width;
            _pictureHeight = height;
            EntityMotorBoat = new EntityMotorBoat();
            EntityMotorBoat.Init(speed, weight, bodyColor, AdditionalColor);
            return true;
        }

        /// <summary> 
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        public void SetPosition(int x, int y)
        {
            if (EntityMotorBoat == null) return;
            while (x + _boatWight > _pictureWight)
            {
                x -= (int)EntityMotorBoat.Step;
            }
            while (x < 0)
            {
                x += (int)EntityMotorBoat.Step;
            }
            while (y + _boatHeight > _pictureHeight)
            {
                y -= (int)EntityMotorBoat.Step;
            }
            while (y < 0)
            {
                y += (int)EntityMotorBoat.Step;
            }
            _startPosX = x;
            _startPosY = y;
        }
        
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(DirectionType direction)
        {
            if (EntityMotorBoat == null)
                return;
            
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    if (_startPosX - EntityMotorBoat.Step > 0)
                    {
                        _startPosX -= (int)EntityMotorBoat.Step;
                    }
                    break;

                //вверх
                case DirectionType.Up:
                    if (_startPosY - EntityMotorBoat.Step > 0)
                    {
                        _startPosY -= (int)EntityMotorBoat.Step;
                    }
                    break;

                //вправо
                case DirectionType.Right:
                    if (_startPosX + _boatWight + EntityMotorBoat.Step < _pictureWight)
                    {
                        _startPosX += (int)EntityMotorBoat.Step;
                    }
                    break;

                //вниз
                case DirectionType.Down:
                    if (_startPosY + _pictureHeight + EntityMotorBoat.Step < _pictureHeight)
                    {
                        _startPosY += (int)EntityMotorBoat.Step;
                    }
                    break;
            }
        }

        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public void DrawTransport(Graphics g)
        {
            if (EntityMotorBoat == null)
            {
                return;
            }
            Pen pen = new(Color.Black);
            Brush additionalBrush = new SolidBrush(EntityMotorBoat.AdditionalColor);

            Point[] points = { new Point(_startPosX + _boatWight - 40, _startPosY + 10), new Point(_startPosX + _boatWight, _startPosY + 40), new Point(_startPosX + _boatWight - 40, _startPosY + 70) };

            g.DrawRectangle(pen, _startPosX, _startPosY, _boatWight - 20, 10);
            g.DrawRectangle(pen, _startPosX + 10, _startPosY + 10, _boatWight - 50, 60);
            g.DrawEllipse(pen, _startPosX + 15, _startPosY + 25, 60, 30); // палуба
            g.DrawPolygon(pen, points);
            

        }
    }
}
